//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_AT_JOB_H
#define LIBPLUMBING_AT_JOB_H

#include <libplumbing/time_value.h>

namespace plumbing {

class at_service; // forward

/**
  * The plumbing::at_job class is used to represent a task to be executed
  * by plumbing::at_service.  The time to be invoked in NOT an attribute of
  * these objects.
  */
class at_job
{
public:
    /**
      * The destructor.
      */
    virtual ~at_job();

protected:
    /**
      * The default construtor.
      * May only be called by derived classes.
      */
    at_job();

public:
    /**
      * The death_subscribe method is used by the
      * plumbing::at_service::register_job method to as to be notified
      * of this objects destruction.  The destructor will call
      * plumbing::at_service::unregister_job in that case.
      */
    void death_subscribe(at_service *sp);

    /**
      * The process method is used to perform the action of the job,
      * when its time has come.
      */
    virtual void process() = 0;

    /**
      * The unregister method is a helper wrapper aroun the
      * plumbing::at_service::unregister_job method.  It assumes that the
      * jobs <i>is</i> actually registered at the moment.
      */
    void unregister();

private:
    /**
      * The death_subscriber instance variable is used to remember the
      * plumbing::at_service which will be interested in hearing about our
      * death.
      */
    at_service *death_subscriber;

    /**
      * The copy constructor.  Do not use.
      */
    at_job(const at_job &);

    /**
      * The assignment operator.  Do not use.
      */
    at_job &operator=(const at_job &);
};

}

#endif // LIBPLUMBING_AT_JOB_H
// vim: set ts=4 et
