//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009, 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libplumbing/config.h>
#include <cstring>
#include <libexplain/write.h>

#include <libplumbing/endpoint/chargen.h>


plumbing::endpoint_chargen::~endpoint_chargen()
{
}


plumbing::endpoint_chargen::endpoint_chargen(int a_fd) :
    endpoint(a_fd),
    sequence(0),
    data_size(0),
    data_position(0)
{
}


plumbing::endpoint::pointer
plumbing::endpoint_chargen::create(int a_fd)
{
    return pointer(new endpoint_chargen(a_fd));
}


void
plumbing::endpoint_chargen::process_read()
{
    // Throw the data away.
    char buffer[1024];
    ssize_t n = ::read(fd, buffer, sizeof(buffer));
    if (n == 0)
        kill_me_now();
}


int
plumbing::endpoint_chargen::get_write_file_descriptor()
{
    return fd;
}


void
plumbing::endpoint_chargen::process_write()
{
    if (data_position >= data_size)
    {
        for (int j = 0; j < 72; ++j)
        {
            data[j] = ' ' + (sequence + j) % 95;
        }
        data[72] = '\r';
        data[73] = '\n';
        ++sequence;
        data_size = 74;
        data_position = 0;
    }
    size_t nbytes = data_size - data_position;
    void *ptr = data + data_position;
    ssize_t n = ::write(fd, ptr, nbytes);
    if (n < 0)
    {
        fatal_error("%s", explain_write(fd, ptr, nbytes));
        kill_me_now();
        return;
    }
    if (n == 0)
    {
        kill_me_now();
        return;
    }
    data_position += n;
}


// vim: set ts=8 sw=4 et
