//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_ENDPOINT_CHARGEN_H
#define LIBPLUMBING_ENDPOINT_CHARGEN_H

#include <boost/shared_ptr.hpp>
#include <libplumbing/endpoint.h>

namespace plumbing {

/**
  * The plumbing::endpoint_chargen class is used to represent
  * a server that gerenerates test as spefified in RFC864.
  */
class endpoint_chargen:
    public endpoint
{
public:
    /**
      * The destructor.
      */
    virtual ~endpoint_chargen();

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param fd
      *     The file descriptor connected to the client for both reading
      *     and writing.
      */
    endpoint_chargen(int fd);

public:
    /**
      * The create class methods is used to create new dynamically
      * allocated instances of this class.  Typically it is used by
      * the plumbing::endpoint_listener_chargen::endpoint_factory method
      * to create new servers when a connection has been accepted.
      *
      * @param fd
      *     The file descriptor connected to the client for both reading
      *     and writing.
      */
    static pointer create(int fd);

protected:
    // See base class for documentation.
    int get_write_file_descriptor();

    // See base class for documentation.
    void process_read();

    // See base class for documentation.
    void process_write();

private:
    /**
      * The sequence instance variable is used to remember
      * the start point for the line cycles we are to generate.
      */
    size_t sequence;

    /**
      * The data instance variable used to remember the data to be
      * transmitted.
      */
    char data[74];

    /**
      * The data_size instance variable is used to remember how much
      * data has been initialized int he data array.
      */
    size_t data_size;

    /**
      * the data_position instance variable is used to remember where we
      * are up to in transmitting the data.
      */
    size_t data_position;

    /**
      * The default constructor.  Do not use.
      */
    endpoint_chargen();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialisation.
      */
    endpoint_chargen(const endpoint_chargen &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The rights hand side of the assignment.
      */
    endpoint_chargen &operator=(const endpoint_chargen &rhs);
};

}

#endif // LIBPLUMBING_ENDPOINT_CHARGEN_H
// vim: set ts=8 sw=4 et
