//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009, 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libplumbing/config.h>
#include <libexplain/write.h>

#include <libplumbing/endpoint/client/stdout.h>
#include <libplumbing/logging.h>


plumbing::endpoint_client_stdout::~endpoint_client_stdout()
{
}


plumbing::endpoint_client_stdout::endpoint_client_stdout(
        const std::string &a_hostname, int a_port_number) :
    endpoint_client(a_hostname, a_port_number)
{
}


plumbing::endpoint::pointer
plumbing::endpoint_client_stdout::create(const std::string &a_hostname,
    int a_port_number)
{
    return pointer(new endpoint_client_stdout(a_hostname, a_port_number));
}


plumbing::endpoint_client_stdout::endpoint_client_stdout(
        const std::string &a_hostname, const std::string &a_port_number) :
    endpoint_client(a_hostname, a_port_number)
{
}


plumbing::endpoint::pointer
plumbing::endpoint_client_stdout::create(const std::string &a_hostname,
    const std::string &a_port_number)
{
    return pointer(new endpoint_client_stdout(a_hostname, a_port_number));
}


void
plumbing::endpoint_client_stdout::data_received(const void *data, size_t nbytes)
{
    if (nbytes == 0)
    {
        kill_me_now();
        return;
    }
    int ofd = fileno(stdout);
    if (ssize_t(nbytes) != ::write(ofd, data, nbytes))
    {
        log().fatal_error("%s", explain_write(ofd, data, nbytes));
        kill_me_now();
    }
}


// vim: set ts=8 sw=4 et
