//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_ENDPOINT_CLIENT_STDOUT_H
#define LIBPLUMBING_ENDPOINT_CLIENT_STDOUT_H

#include <libplumbing/endpoint/client.h>

namespace plumbing {

/**
  * The plumbing::endpoint_client_stdout class is used to represent a
  * plumbing::reactor client that simply prints all received data on the
  * standard output.
  */
class endpoint_client_stdout:
    public endpoint_client
{
public:
    /**
      * The destructor.
      */
    virtual ~endpoint_client_stdout();

private:
    /**
      * A constructor.
      * It is private on purpose, use a #create class method instead.
      *
      * @param hostname
      *     The name of the remote host.  May be a dotted-quad or a
      *     string to be looked-up via gethostbyname.
      * @param port
      *     The port number.
      */
    endpoint_client_stdout(const std::string &hostname, int port);

public:
    /**
      * The create class method id used to create new dynamically
      * allocated instances of this class.
      *
      * @param hostname
      *     The name of the remote host.  May be a dotted-quad or a
      *     string to be looked-up via gethostbyname.
      * @param port
      *     The port number.
      */
    static pointer create(const std::string &hostname, int port);

private:
    /**
      * A constructor.
      *
      * @param hostname
      *     The name of the remote host.  May be a dotted-quad or a
      *     string to be looked-up via gethostbyname.
      * @param port
      *     The port number.  May be a string service name ("ssh") or a
      *     numeric port number ("22").
      */
    endpoint_client_stdout(const std::string &hostname,
        const std::string &port);

public:
    /**
      * The create class method id used to create new dynamically
      * allocated instances of this class.
      *
      * @param hostname
      *     The name of the remote host.  May be a dotted-quad or a
      *     string to be looked-up via gethostbyname.
      * @param port
      *     The port number.  May be a string service name ("ssh") or a
      *     numeric port number ("22").
      */
    static pointer create(const std::string &hostname, const std::string &port);

protected:
    // See base class for documentation.
    void data_received(const void *data, size_t nbytes);

private:
    /**
      * The default constructor.  Do not use.
      */
    endpoint_client_stdout();

    /**
      * The copy constructor.  Do not use.
      */
    endpoint_client_stdout(const endpoint_client_stdout &);

    /**
      * The assignment operator.  Do not use.
      */
    endpoint_client_stdout &operator=(const endpoint_client_stdout &);
};

}

#endif // LIBPLUMBING_ENDPOINT_CLIENT_STDOUT_H
// vim: set ts=8 sw=4 et
