//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009, 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libplumbing/config.h>
#include <cstring>
#include <libexplain/write.h>

#include <libplumbing/endpoint/daytime.h>


plumbing::endpoint_daytime::~endpoint_daytime()
{
}


plumbing::endpoint_daytime::endpoint_daytime(int a_fd) :
    endpoint(a_fd),
    data_position(0),
    data_size(0)
{
    time_t now;
    time(&now);
    struct tm *tmp = localtime(&now);
    strftime(data, sizeof(data), "%c\n", tmp);
    data_size = strlen(data);
}


plumbing::endpoint::pointer
plumbing::endpoint_daytime::create(int a_fd)
{
    return pointer(new endpoint_daytime(a_fd));
}


void
plumbing::endpoint_daytime::process_read()
{
    // Throw the data away.
    char buffer[1024];
    ssize_t n = ::read(fd, buffer, sizeof(buffer));
    if (n == 0)
        kill_me_now();
}


int
plumbing::endpoint_daytime::get_write_file_descriptor()
{
    return fd;
}


void
plumbing::endpoint_daytime::process_write()
{
    if (data_position >= data_size)
    {
        kill_me_now();
    }
    else
    {
        size_t nbytes = data_size - data_position;
        void *ptr = data + data_position;
        ssize_t n = ::write(fd, ptr, nbytes);
        if (n < 0)
            fatal_error("%s", explain_write(fd, ptr, nbytes));
        data_position += n;
    }
}


// vim: set ts=8 sw=4 et
