//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_ENDPOINT_DAYTIME_H
#define LIBPLUMBING_ENDPOINT_DAYTIME_H

#include <libplumbing/endpoint.h>

namespace plumbing {

/**
  * The plumbing::endpoint_daytime class is used to represent
  * a server that reports the time-of-day in text.
  */
class endpoint_daytime:
    public endpoint
{
public:
    /**
      * The destructor.
      */
    virtual ~endpoint_daytime();

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param fd
      *     The file descriptor connected to the client for both reading
      *     and writing.
      */
    endpoint_daytime(int fd);

public:
    /**
      * The create class methods is used to create new dynamically
      * allocated instances of this class.  Typically it is used by
      * the plumbing::endpoint_listener_daytime::endpoint_factory method
      * to create new servers when a connection has been accepted.
      *
      * @param fd
      *     The file descriptor connected to the client for both reading
      *     and writing.
      */
    static pointer create(int fd);

protected:
    // See base class for documentation.
    int get_write_file_descriptor();

    // See base class for documentation.
    void process_read();

    // See base class for documentation.
    void process_write();

private:
    /**
      * The data instance variable is used to remember the data we wish
      * to send to the client.
      */
    char data[100];

    /**
      * The data_position instance variable is used to remember how much
      * of the data has been sent, so far.
      */
    size_t data_position;

    /**
      * The data_size instance variable is used to remember how much of
      * the data has been used.  This is the limit of the #data_position
      * instance variable.
      */
    size_t data_size;

    /**
      * The default constructor.  Do not use.
      */
    endpoint_daytime();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialisation.
      */
    endpoint_daytime(const endpoint_daytime &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The rights hand side of the assignment.
      */
    endpoint_daytime &operator=(const endpoint_daytime &rhs);
};

}

#endif // LIBPLUMBING_ENDPOINT_DAYTIME_H
// vim: set ts=8 sw=4 et
