//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_ENDPOINT_FUNCTOR_H
#define LIBPLUMBING_ENDPOINT_FUNCTOR_H

#include <boost/shared_ptr.hpp>
#include <libplumbing/endpoint.h>

namespace plumbing {

/**
  * The plumbing::endpoint_functor class is used to represent and abstract
  * operator to be performed on a plumbing::endpoint instance.  It is
  * typically handed to the plumbing::reactor::for_each_endpoint method.
  */
class endpoint_functor
{
public:
    /**
      * The destructor.
      */
    virtual ~endpoint_functor();

    /**
      * The default constructor.
      */
    endpoint_functor();

    /**
      * The copy constructor.
      */
    endpoint_functor(const endpoint_functor &);

    /**
      * The assignment operator.
      */
    endpoint_functor &operator=(const endpoint_functor &);

    /**
      * The function call operator is used to perform the action
      * associated with this functor.
      */
    virtual void operator()(const endpoint::pointer &sp) = 0;
};

}

#endif // LIBPLUMBING_ENDPOINT_FUNCTOR_H
// vim: set ts=4 et
