//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_ENDPOINT_LISTENER_DAYTIME_H
#define LIBPLUMBING_ENDPOINT_LISTENER_DAYTIME_H

#include <libplumbing/endpoint/listener.h>

namespace plumbing {

/**
  * The plumbing::endpoint_listener_daytime class is used to
  * represent a socket that listens for connections, and spawns
  * plumbing::endpoint_daytime objects to serve those new connections.
  */
class endpoint_listener_daytime:
    public endpoint_listener
{
public:
    /**
      * The destructor.
      */
    virtual ~endpoint_listener_daytime();

private:
    /**
      * The constructor.
      * It is private on purpse, use the #create class method instead.
      *
      * @param svc
      *     The reactor to manage all new connections.
      * @param port_number
      *     The numer of the port to use, or the empty string if it should
      *     use the well known port.
      * @param repeat_count
      *     The number of time to spawn servers before self terminating.
      *     Use -1 to mean "infinity".
      */
    endpoint_listener_daytime(reactor &svc, const std::string &port_number,
        int repeat_count = -1);

public:
    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param svc
      *     The reactor to manage all new connections.
      * @param port_number
      *     The numer of the port to use, or the empty string if it should
      *     use the well known port.
      * @param repeat_count
      *     The number of time to spawn servers before self terminating.
      *     Use -1 to mean "infinity".
      */
    static pointer create(reactor &svc, const std::string &port_number = "",
        int repeat_count = -1);

protected:
    // See base class for documentation.
    int get_default_port() const;

    // See base class for documentation.
    endpoint::pointer endpoint_factory(int fd, void *adr, int adrlen);

private:
    /**
      * The default constructor.  Do not use.
      */
    endpoint_listener_daytime();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialisation.
      */
    endpoint_listener_daytime(const endpoint_listener_daytime &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The rights hand side of the assignment.
      */
    endpoint_listener_daytime &operator=(const endpoint_listener_daytime &rhs);
};

}

#endif // LIBPLUMBING_ENDPOINT_LISTENER_DAYTIME_H
// vim: set ts=8 sw=4 et
