//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <libplumbing/logging.h>
#include <libplumbing/logging/stderr.h>


plumbing::logging *plumbing::logging::singleton;


plumbing::logging::~logging()
{
    if (singleton == this)
        singleton = 0;
}


plumbing::logging::logging()
{
}


void
plumbing::logging::message(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    message_v(fmt, ap);
    va_end(ap);
}


void
plumbing::logging::message_v(const char *fmt, va_list ap)
{
    char buffer[2000];
    vsnprintf(buffer, sizeof(buffer), fmt, ap);
    deliver_message_text(buffer);
}


void
plumbing::logging::warning(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    warning_v(fmt, ap);
    va_end(ap);
}


void
plumbing::logging::warning_v(const char *fmt, va_list ap)
{
    char buffer[2000];
    vsnprintf(buffer, sizeof(buffer), fmt, ap);
    message("warning: %s", buffer);
}


void
plumbing::logging::fatal_error(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    fatal_error_v(fmt, ap);
    va_end(ap);
}


void
plumbing::logging::fatal_error_v(const char *fmt, va_list ap)
{
    char buffer[2000];
    vsnprintf(buffer, sizeof(buffer), fmt, ap);
    deliver_message_text(buffer);
    exit();
}


void
plumbing::logging::exit()
{
    ::exit(1);
}


void
plumbing::logging::register_stream(logging *ls)
{
    singleton = ls;
}


plumbing::logging &
plumbing::logging::get_singleton()
{
    if (!singleton)
    {
        static logging_stderr xerr;
        singleton = &xerr;
    }
    return *singleton;
}


// vim: set ts=4 et
