//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libplumbing/config.h>
#include <cerrno>
#include <cstring>
#include <libexplain/fopen.h>
#include <libexplain/fflush.h>

#include <libplumbing/logging/file.h>
#include <libplumbing/progname.h>


plumbing::logging_file::~logging_file()
{
    if (vfp)
    {
        FILE *fp = (FILE *)vfp;
        vfp = 0;
        fclose(fp);
    }
}


plumbing::logging_file::logging_file(const std::string &a_filename) :
    logging(),
    filename(a_filename),
    vfp(0),
    since(0)
{
}


void
plumbing::logging_file::deliver_message_text(const char *txt)
{
    if (vfp)
    {
        time_t now;
        time(&now);
        if (now - since >= 60 * 60)
        {
            //
            // Close the file every hour.
            // This makes the logrotate(8) simple.
            //
            FILE *fp = (FILE *)vfp;
            vfp = 0;
            fclose(fp);
        }
    }

    FILE *fp = (FILE *)vfp;
    if (!fp)
    {
        fp = fopen(filename.c_str(), "a");
        if (!fp)
        {
            int err = errno;

            //
            // We can't log it (via plumbing::log.fatal_error) because
            // this code *is* the logging.  Catch 22.  So we use stderr.
            //
            fprintf
            (
                stderr,
                "%s: %s\n",
                progname_get().c_str(),
                explain_errno_fopen(err, filename.c_str(), "a")
            );
            fp = stderr;
        }
        vfp = fp;
    }

    fprintf(fp, "%s\n", txt);
    if (fflush(fp))
    {
        int err = errno;
        fprintf
        (
            stderr,
            "%s: %s\n",
            progname_get().c_str(),
            explain_errno_fflush(err, fp)
        );
        fclose(fp);
        vfp = 0;
    }
}


// vim: set ts=8 sw=4 et
