//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_LOGGING_FILE_H
#define LIBPLUMBING_LOGGING_FILE_H

#include <ctime>
#include <string>

#include <libplumbing/logging.h>

namespace plumbing {

/**
  * The plumbing::logging_file class is used to represent a logging
  * destination which is a file.
  *
  * The file will be closed and reopened periodically to facilitate
  * logrotate(8) and friends.
  */
class logging_file:
    public logging
{
public:
    /**
      * The destructor.
      */
    virtual ~logging_file();

    /**
      * The constructor.
      *
      * @param filename
      *     the name of the file which is to receive the logging data.
      */
    logging_file(const std::string &filename);

protected:
    // See base class fopr documentation.
    void deliver_message_text(const char *txt);

private:
    /**
      * The filename isntance variable is sued to rememeber the name of
      * the file which is to receive the logging data.
      */
    std::string filename;

    /**
      * The vfp instance variable is used to remember the open file.  It
      * will be NULL when the file is not open.
      *
      * @note
      *    The only reason it's a void* and not a FILE* is to avoid
      *    sucking in stdio.h
      */
    void *vfp;

    /**
      * The since instance variable is used to remember the time we last
      * opened the file.  This is used when deciding whether or not to
      * close and reopen the file, to make logrotate(8) easy.
      */
    time_t since;

    /**
      * The default constructor.  Do not use.
      */
    logging_file();

    /**
      * The copy constructor.  Do not use.
      */
    logging_file(const logging_file &);

    /**
      * The assignment operator.  Do not use.
      */
    logging_file &operator=(const logging_file &);
};

}

#endif // LIBPLUMBING_LOGGING_FILE_H
// vim: set ts=8 sw=4 et
