//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_LOGGING_NULL_H
#define LIBPLUMBING_LOGGING_NULL_H

#include <libplumbing/logging.h>

namespace plumbing {

/**
  * The plumbing::logging_null class is used to represent
  * a logging destination that throws everything away.
  */
class logging_null:
    public logging
{
public:
    /**
      * The destructor.
      */
    virtual ~logging_null();

    /**
      * The default constructor.
      */
    logging_null();

protected:
    // See base class for documentation.
    void deliver_message_text(const char *txt);

private:
    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialisation.
      */
    logging_null(const logging_null &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The rights hand side of the assignment.
      */
    logging_null &operator=(const logging_null &rhs);
};

}

#endif // LIBPLUMBING_LOGGING_NULL_H
// vim: set ts=8 sw=4 et
