//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstring>
#include <syslog.h>

#include <libplumbing/logging/syslog.h>
#include <libplumbing/progname.h>


plumbing::logging_syslog::~logging_syslog()
{
    if (is_open)
    {
        closelog();
        is_open = false;
    }
}


plumbing::logging_syslog::logging_syslog() :
    logging(),
    is_open(false)
{
}


static int
determine_level_from_text(const char *txt)
{
    struct table_t
    {
        const char *name;
        int level;
    };

    static const table_t table[] =
    {
        { "emerg", LOG_EMERG },
        { "emergency", LOG_EMERG },
        { "alert", LOG_ALERT },
        { "crit", LOG_CRIT },
        { "critical", LOG_CRIT },
        { "err", LOG_ERR },
        { "error", LOG_ERR },
        { "warn", LOG_WARNING },
        { "warning", LOG_WARNING },
        { "notice", LOG_NOTICE },
        { "info", LOG_INFO },
        { "information", LOG_INFO },
        { "debug", LOG_DEBUG },
        { 0, 0 }
    };

    for (const table_t *tp = table; tp->name; ++tp)
    {
        size_t len = strlen(tp->name);
        if (strncasecmp(tp->name, txt, len) == 0 && txt[len] == ':')
            return tp->level;
    }
    return LOG_ERR;
}


void
plumbing::logging_syslog::deliver_message_text(const char *txt)
{
    int facility = LOG_USER;
    if (!is_open)
    {
        int option = LOG_PID;
        openlog(progname_get().c_str(), option, facility);
    }
    int level = determine_level_from_text(txt);
    int priority = facility | level;
    syslog(priority, "%s\n", txt);
}


// vim: set ts=8 sw=4 et
