//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_LOGGING_SYSLOG_H
#define LIBPLUMBING_LOGGING_SYSLOG_H

#include <libplumbing/logging.h>

namespace plumbing {

/**
  * The plumbing::logging_syslog class is used to represent a loging stream
  * which is directed into the system log (syslog(8)) facility.
  */
class logging_syslog:
    public logging
{
public:
    /**
      * The destructor.
      */
    virtual ~logging_syslog();

    /**
      * The default constructor.
      */
    logging_syslog();

protected:
    // See base class for documentation.
    void deliver_message_text(const char *txt);

private:
    /**
      * The is_open instance variable is used to remember whether or not
      * the openlog() function has been called to start logging (and
      * open the connection to syslogd).
      */
    bool is_open;

    /**
      * The copy constructor.  Do not use.
      */
    logging_syslog(const logging_syslog &);

    /**
      * The assignment operator.  Do not use.
      */
    logging_syslog &operator=(const logging_syslog &);
};

}

#endif // LIBPLUMBING_LOGGING_SYSLOG_H
// vim: set ts=8 sw=4 et
