//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBPLUMBING_LOGGING_TEE_H
#define LIBPLUMBING_LOGGING_TEE_H

#include <libplumbing/logging.h>

namespace plumbing {

/**
  * The plumbing::logging_tee class is used to represent a logging stream
  * which can deliver the messages to two destinations simultaneously.
  */
class logging_tee:
    public logging
{
public:
    /**
      * The destructor.
      */
    virtual ~logging_tee();

    /**
      * The constructor.
      */
    logging_tee(logging *lhs, logging *rhs);

protected:
    // See base class for documentation.
    void deliver_message_text(const char *txt);

private:
    /**
      * The lhs instance variable is used to remember the first logging
      * stream.
      */
    logging *lhs;

    /**
      * The rhs instance variable is used to remember the second logging
      * stream.
      */
    logging *rhs;

    /**
      * The default constructor.  Do not use.
      */
    logging_tee();

    /**
      * The copy constructor.  Do not use.
      */
    logging_tee(const logging_tee &);

    /**
      * The assignment operator.  Do not use.
      */
    logging_tee &operator=(const logging_tee &);
};

}

#endif // LIBPLUMBING_LOGGING_TEE_H
// vim: set ts=8 sw=4 et
