//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cassert>
#include <cstring>

#include <libplumbing/progname.h>


static std::string progname;


void
plumbing::progname_set(const char *s)
{
    for (;;)
    {
        const char *cp1 = strchr(s, '/');
        if (!cp1)
        {
            progname = s;
            return;
        }
        const char *cp2 = cp1 + 1;
        while (*cp2 == '/')
            ++cp2;
        if (!*cp2)
        {
            progname = std::string(s, cp1 - s);
            return;
        }
        s = cp2;
    }
}


std::string
plumbing::progname_get()
{
    if (progname.empty())
    {
        char buf[1000];
        int n = readlink("/proc/self/exe", buf, sizeof(buf) - 1);
        if (n > 0)
        {
            buf[n] = 0;
            progname_set(buf);
        }
        else
            progname_set("???");
    }

    assert(!progname.empty());
    return progname;
}


// vim: set ts=8 sw=4 et
