//
// plumbing - C++ wrapper facades for Berkeley sockets
// Copyright (C) 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License, version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <cstdlib>
#include <unistd.h>

#include <libplumbing/reactor.h>
#include <libplumbing/endpoint/client/stdout.h>
#include <libplumbing/logging.h>
#include <libplumbing/progname.h>
#include <libplumbing/version_print.h>


static void
usage()
{
    std::string prog = plumbing::progname_get();
    fprintf(stderr, "Usage: %s <hostname> <port>\n", prog.c_str());
    fprintf(stderr, "       %s -V\n", prog.c_str());
    exit(1);
}


int
main(int argc, char **argv)
{
    plumbing::progname_set(argv[0]);
    for (;;)
    {
        int c = getopt(argc, argv, "V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'V':
            plumbing::version_print();
            return 0;

        default:
            usage();
        }
    }
    std::string hostname;
    std::string port;
    switch (argc - optind)
    {
    case 2:
        hostname = argv[optind];
        port = argv[optind + 1];
        break;

    default:
        usage();
        // NOTREACHED
    }

    //
    // This is the simplest main loop that can be achieved with the
    // Plumbing library.
    //
    // The reactor is instantiated, the endpoint is created, and the
    // reactor is told to manage the endpoint.
    //
    // The reactor then runs until there are no more endpoints to be
    // managed.  When they are done, endpoints remove themselves from
    // the select loop.
    //
    plumbing::reactor fat_controller;
    plumbing::endpoint::pointer thomas =
        plumbing::endpoint_client_stdout::create(hostname, port);
    fat_controller.add_endpoint(thomas);
    fat_controller.run();
    return 0;
}


// vim: set tw=8 sw=4 et
